/*********************************************************

INCLUDE FILE: parameters.agc
  by Steve Vink, March 2015
  Use permitted for all purposes, commercial and personal

  These functions perform the following:
  Loading parameter files
  Saving parameter files
  Getting parameters
  Setting parameters
  
  The parameter file format is:
  
    parameter=value
  
  e.g
  
    appname=My Application
    screenwidth=1440
    screenheight=900
  
**********************************************************/


//*********************************************************
//  TYPE: 	tParameter
//	      	used for the storage of parameters loaded from files
//    
//			parm is the parameter name
//			value is the parameter value
//			id is the parameter file id
//*********************************************************

type tParm
    parm as string
    value as string
    id as string
endtype

global arrParm as tParm[]
global gParameterCount

//*********************************************************
  function initParms(numParms as integer)
//*********************************************************
//	Initialise the parameter subsystem
//
//  numParms is the starting number of parameters
//  This will be expanded if necessary by the program
//  It does not have to be exact
//*********************************************************
	
	arrParm.length = numParms

endfunction


//*********************************************************
  function LoadParms(file$,id$)
//*********************************************************
// Load a set of parameters from a file.
// Supply the file name, and the text ID by which you will
// refer to these parameters. 
// For example, "game", "player" or "scores"
// You can load multiple files into the paramter list,
// by specifying a different text ID.
//*********************************************************

	//ensure the file exists
    if getFileExists(file$) = 0
        exitfunction
    endif

	//open the file and read line by line
    f = openToRead(file$)
    while fileEOF(f) = 0
        in$ = readLine(f)
        //ignore commented line befos (*)
        if left(in$,1) = "*"
			continue
		endif
        // Check the string contains "="
        txt$ = getStringToken(in$,"=",2)
        if txt$ <> ""
            success = 0
            parm$ = lower(getStringToken(in$,"=",1))
			//check if parameter is already in the array
			//if it is, overwrite with the new value
            for n = 1 to gParameterCount
                if arrParm[gParameterCount].id = id$
                    if arrParm[gParameterCount].parm = parm$
                        arrParm[n].value = txt$
                        success = 1
                        exit
                    endif
                endif
            next n

			//if this is a new parameter (success from previous step = 0), add it to the array
            if success = 0
                // Increment Parameter Count. If > number of elements, add 10 more array elements
                gParameterCount = gParameterCount + 1
                if gParameterCount > arrParm.length
                    arrParm.length = gParameterCount + 9
                endif
                arrParm[gParameterCount].id = id$    //
                arrParm[gParameterCount].value = txt$
                arrParm[gParameterCount].parm = lower(getStringToken(in$,"=",1))
            endif
        endif
    endwhile
  
endfunction

//*********************************************************
  function getParm(parm$, id$)   
//*********************************************************
// Request the value of a parameter by providing the 
// parameter name and the file identifier
// All parameters are string values
// If you want an integer or float value then convert them:
// 
// MyVal = val(getParm(parm$, id$))
// MyVal# = valFloat(getParm(parm$, id$))
//*********************************************************

	//convert request to lowercase and initialise the return value
    parm$ = lower(parm$)
    id$ = lower(id$)
    ret$ = ""

	// loop through the parameters and verify parameter name anf file identifier
	// if nothing is found, the default blank value will be returned
    for n = 1 to gParameterCount
        if parm$ = arrParm[n].parm
            if id$ = arrParm[n].id    
                ret$ = arrParm[n].value
                exit
            endif    
        endif
    next n

endfunction ret$

//*********************************************************
  function setParm(parm$, value$, id$)
//*********************************************************
// Set the value of a parameter, either new or existing
// if new, it will be added to the array of parameters
// it can be saved to the identified file (id$) later
//  using saveParms()
//
// numeric values are saved as strings and must be converted
// before calling this function using str(myValue)
//*********************************************************

	// convert parameter name to lowercase
    parm$ = lower(parm$)
    id$ = lower(id$)
    
	// Loop through  parameters
	// if found, update
    for n = 1 to gParameterCount
        if parm$ = arrParm[n].parm
            if id$ = arrParm[n].id
                arrParm[n].value = value$
                success = 1
                exit
            endif
        endif
    next n
    
    // if not found in prvious step (success = 0) then create parameter
    if success = 0
        // Increment Parameter Count. If exceeded add 10 more array elements   
        gParameterCount = gParameterCount + 1
		// if array elements exceeded, add 10 more
		if gParameterCount > arrParm.length
			arrParmlength = gParameterCount + 9
		endif
        arrParm[gParameterCount].parm = parm$
        arrParm[gParameterCount].value = value$
        arrParm[gParameterCount].id = id$
    endif

endfunction


//*********************************************************
  function SaveParms(file$, id$)
//*********************************************************
// Save the parameters to the parameter file.
// Any new or modified parameters will be updated
// Remember, the new file will be in the writable folder!
// Any manual changes to the original file in the media folder
// will be ignored after saving the file
// Any comments (*) in the file are not written back.
//*********************************************************

	// Delete the file if it exists, so we can recreate it
    if getFileExists(file$) = 1
        deleteFile(file$)
    endif

	// Open the file and write the parameters to it.
    f = openToWrite(file$,0)
    for n = 1 to gParameterCount
        if arrParm[n].id = id$    
            writeLine(f,arrParm[n].parm + "=" + arrParm[n].value)
        endif   //
    next n
    closeFile(f)

endfunction

