/*********************************************************

INCLUDE FILE: shake.agc
  by Steve Vink, June 2015
  Use permitted for all purposes, commercial and personal

  These functions perform the following:
  Initialising the Screen Shake module
  Triggering a new shake
  Updating the Shake
  
**********************************************************/


//*********************************************************
//  TYPE: 	tShake
//	      	used for the shake parameters
//    
//			duration - length of a shake
//			count - number of shakes
//			scale - Size of initial shake
//			interval - calculated interval between shakes
//			tween - movement tween
//			chain - tweenchain
//			sprite - hidden sprite to control the movement
//*********************************************************
type tShake
	active as integer
	duration as float
	count as integer
	scale as float
	interval as float
	tween as integer[]
	chain as integer
	sprite as integer
endtype

// Type to store X/Y Values
type tVector
	x as float
	y as float
endtype

global gShake as tShake

//**** Set up some data arrays
// The first array is a set of X and Y position multipliers that represent 12 points arount the circle centre-point
global arrShakeVector as tVector[12]

// The second array stores a random set of directions.
// This is twice as big as the number of shakes required, to allow for the perception of randomness
global arrShakePoint as integer[]

// The third array stores the magnitude of each shake
global arrShakeMagnitude as float[]


//*********************************************************
function initShake(duration as float, count as integer, scale as float)
//	Initialisation of Shake Module
//	Pass in the shake duration, number of shakes and magnitude (pixels)
//	
//	The shake vectors (directions) will be initialised
//	The random directions will be stored
//	The magnitude of each shake will be calculated and stored
//*********************************************************
	
		gShake.duration = duration
		gShake.count = Count
		gShake.scale = scale
		gShake.tween.length = Count
		gShake.active = 0
		
		//Create the Vectors for shake direction
		for n = 0 to 359 step 30
			e = n / 30
			arrShakeVector[e].x = cos(n)
			arrShakeVector[e].y = sin(n)
		next n

		//Create the random positions. Make sure that every position is on the opposite hemisphere
		arrShakePoint.length = 40
		arrshakePoint[0] = random(1,12)
		for n = 1 to 40
			arrshakePoint[n] = random(arrshakePoint[n] + 3,arrshakePoint[n] + 9)
			if arrshakePoint[n] > 12 then dec arrshakePoint[n], 12
		next n
		
		//Now calculate the magnitudes of each shake. Start at 1 and reduce to 0
		arrShakeMagnitude.length = count
		for n = 1 to Count
			a# = n/(count * 1.0) * 90.0
			arrShakeMagnitude[n] = cos(a#)
		next n
		
		//Set the tweens
		gShake.chain = CreateTweenChain()
		dur# = gShake.duration / gShake.count
		for n = 1 to count
			gShake.tween[n] = CreateTweenSprite(dur#)
		next n
		
		//Create the Sprite
		gShake.sprite = CreateSprite(0)
		SetSpriteVisible(gShake.sprite, 0)

endfunction

//*********************************************************
function ShakeStart()
//	Start the shaking!
// 	If already shaking, reset
// 	Set up the shake tween chain
//*********************************************************

		gShake.active = 1
		log("Shake Started")
		if GetTweenChainPlaying(gShake.chain) = 1
			StopTweenChain(gShake.chain)
		endif
		DeleteTweenChain(gShake.chain)
		CreateTweenChain(gShake.chain)
		//ClearTweenChain(gShake.chain)
		sync()

		//Pick a starting vector, to randomise the effect
		vector = random(1,40)
		
		x# = 0: y# = 0
		for n = 1 to gShake.count
			newx# = arrShakeVector[arrShakePoint[vector]].x * arrShakeMagnitude[n] * gShake.scale
			newy# = arrShakeVector[arrShakePoint[vector]].y * arrShakeMagnitude[n] * gShake.scale
			SetTweenSpriteX(gShake.tween[n], x#,newx# ,TweenSmooth1())
			SetTweenSpriteY(gShake.tween[n], y#, newy#,TweenSmooth1())
			AddTweenChainSprite(gShake.chain,gShake.tween[n],gShake.sprite,0)
			x# = newx#: y# = newy#
			inc vector,1
			if vector > 40 then vector = 1
		next n
		PlayTweenChain(gShake.chain)
		sync()

endfunction


//*********************************************************
function ShakeUpdate()
//	Check Shake status and act accordingly
//*********************************************************

	if gShake.active = 0
		exitfunction
	endif
	
	if GetTweenChainPlaying(gShake.chain) = 0
		gShake.active = 0
		SetViewOffset(0,0)
		log("Shake Stopped")
		exitfunction
	endif
	
	SetViewOffset(getSpriteX(gShake.sprite), getSpriteY(gShake.sprite))
	
endfunction
